/*
 *  Driver for the ViewCast 845e PCIe Capture Card
 *
 *  Copyright 2011/2012, Kernel Labs Inc. www.kernellabs.com.
 *   - Steven Toth <stoth@kernellabs.com>
 *
 *  Project sponsored by ViewCast Texas, US.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "vc8x0.h"

static unsigned int vc8x0_ad7441_debug;
module_param_named(ad7441_debug, vc8x0_ad7441_debug, int, 0644);
MODULE_PARM_DESC(ad7441_debug, "enable debug messages [ad7441]");

#define dprintk(level, fmt, arg...)\
	do { if (vc8x0_ad7441_debug >= level)\
		printk(KERN_ERR " " fmt, ## arg);\
	} while (0)

static int vc8x0_ad7441_HDMI_1920x1080px60(struct vc8x0_ad7441_context *ctx, int tdmsfix);

int vc8x0_ad7441_set_edid(struct vc8x0_ad7441_context *ctx, u8 *data, int len);

int vc8x0_ad7441_sw_reset(struct vc8x0_ad7441_context *ctx);

/* VIEWCAST EDID  - Used from 01/10/12 */
static u8 edid_data[256] =
{
0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x06, 0x96, 0x00, 0x08, 0x01, 0x00, 0x00, 0x00, 
0x0F, 0x14, 0x01, 0x04, 0x80, 0x31, 0x1C, 0xA0, 0x2A, 0xAA, 0x33, 0xA4, 0x55, 0x48, 0x93, 0x25, 
0x10, 0x45, 0x47, 0x2F, 0xCF, 0x00, 0x31, 0x59, 0x45, 0x59, 0x61, 0x59, 0x81, 0x80, 0xA9, 0x40, 
0xD1, 0x00, 0xB3, 0x00, 0x95, 0x00, 0x02, 0x3A, 0x80, 0x18, 0x71, 0x38, 0x2D, 0x40, 0x58, 0x2C, 
0x45, 0x00, 0xE8, 0x12, 0x11, 0x00, 0x00, 0x1E, 0x00, 0x00, 0x00, 0xFD, 0x00, 0x17, 0x56, 0x0F, 
0x6F, 0x11, 0x04, 0x10, 0x00, 0x00, 0xF0, 0x38, 0x00, 0x01, 0x00, 0x00, 0x00, 0xFC, 0x00, 0x4F, 
0x73, 0x70, 0x72, 0x65, 0x79, 0x38, 0x30, 0x30, 0x20, 0x31, 0x2E, 0x34, 0x00, 0x00, 0x00, 0xF7, 
0x00, 0x0A, 0x1F, 0xFF, 0xFF, 0x64, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xED, 
0x02, 0x03, 0x2A, 0x72, 0x55, 0x90, 0x05, 0x1F, 0x14, 0x04, 0x13, 0x01, 0x02, 0x03, 0x11, 0x12, 
0x06, 0x07, 0x15, 0x16, 0x22, 0x21, 0x20, 0x3E, 0x3D, 0x3C, 0x23, 0x0F, 0x57, 0x07, 0x83, 0x4F, 
0x00, 0x00, 0x67, 0x03, 0x0C, 0x00, 0x10, 0x00, 0xA8, 0x2D, 0x01, 0x1D, 0x80, 0x18, 0x71, 0x1C, 
0x16, 0x20, 0x58, 0x2C, 0x25, 0x00, 0xE8, 0x12, 0x11, 0x00, 0x00, 0x9E, 0x01, 0x1D, 0x80, 0xD0, 
0x72, 0x1C, 0x16, 0x20, 0x10, 0x2C, 0x25, 0x80, 0xE8, 0x12, 0x11, 0x00, 0x00, 0x9E, 0x01, 0x1D, 
0x00, 0x72, 0x51, 0xD0, 0x1E, 0x20, 0x6E, 0x28, 0x55, 0x00, 0xE8, 0x12, 0x11, 0x00, 0x00, 0x1E, 
0x01, 0x1D, 0x00, 0xBC, 0x52, 0xD0, 0x1E, 0x20, 0xB8, 0x28, 0x55, 0x40, 0xE8, 0x12, 0x11, 0x00, 
0x00, 0x1E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x91, 
};

static int vc8x0_ad7441_i2c_usermap_read8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 *val)
{
	int ret;

	ret = vc8x0_I2cReadRegister(ctx->bus, ctx->usermap_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_usermap_write8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 val)
{
	int ret;

	ret = vc8x0_I2cWriteRegister(ctx->bus, ctx->usermap_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_usermap_setbit(struct vc8x0_ad7441_context *ctx, u8 reg, u8 bitmask)
{
	unsigned char val;
	vc8x0_ad7441_i2c_usermap_read8(ctx, reg, &val);
	val |= bitmask;
	return vc8x0_ad7441_i2c_usermap_write8(ctx, reg, val);
}

static int vc8x0_ad7441_i2c_user1map_read8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 *val)
{
	int ret;

	ret = vc8x0_I2cReadRegister(ctx->bus, ctx->user1map_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_user1map_write8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 val)
{
	int ret;

	ret = vc8x0_I2cWriteRegister(ctx->bus, ctx->user1map_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_user1map_setbit(struct vc8x0_ad7441_context *ctx, u8 reg, u8 bitmask)
{
	unsigned char val;
	vc8x0_ad7441_i2c_user1map_read8(ctx, reg, &val);
	val |= bitmask;
	return vc8x0_ad7441_i2c_user1map_write8(ctx, reg, val);
}

static int vc8x0_ad7441_i2c_user2map_write8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 val)
{
	int ret;

	ret = vc8x0_I2cWriteRegister(ctx->bus, ctx->user2map_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_hdmimap_read8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 *val)
{
	int ret;

	ret = vc8x0_I2cReadRegister(ctx->bus, ctx->hdmimap_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_hdmimap_write8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 val)
{
	int ret;

	ret = vc8x0_I2cWriteRegister(ctx->bus, ctx->hdmimap_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_hdmimap_setbit(struct vc8x0_ad7441_context *ctx, u8 reg, u8 bitmask)
{
	unsigned char val;
	vc8x0_ad7441_i2c_usermap_read8(ctx, reg, &val);
	val |= bitmask;
	return vc8x0_ad7441_i2c_hdmimap_write8(ctx, reg, val);
}

static int vc8x0_ad7441_i2c_edidmap_write8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 val)
{
	int ret;

	ret = vc8x0_I2cWriteRegister(ctx->bus, ctx->edidmap_addr, reg, val);
	return ret;
}

static int vc8x0_ad7441_i2c_rksvmap_write8(struct vc8x0_ad7441_context *ctx, u8 reg, u8 val)
{
	int ret;

	ret = vc8x0_I2cWriteRegister(ctx->bus, ctx->rksvmap_addr, reg, val);
	return ret;
}

int vc8x0_ad7441_init(struct vc8x0_dma_channel *dma_channel, u8 i2caddr)
{
	struct vc8x0_ad7441_context *ctx = &dma_channel->ad7441_ctx;

	dprintk(1, "%s(%p, 0x%x)\n", __func__, dma_channel, i2caddr);
	memset(ctx, 0, sizeof(*ctx));

	ctx->dma_channel = dma_channel;
	ctx->bus = &dma_channel->i2c_bus;
	ctx->usermap_addr = i2caddr;
	ctx->channel = 0;

	if (ctx->usermap_addr == 0x42) {
		ctx->user1map_addr	= 0x46;
		ctx->user2map_addr	= 0x62;
		ctx->user3map_addr	= 0x4a;
		ctx->reservedmap_addr	= 0x4e;
		ctx->hdmimap_addr	= 0x6a;
		ctx->rksvmap_addr	= 0x66;
		ctx->edidmap_addr	= 0x6e;
	} else
	if (ctx->usermap_addr == 0x40) {
		ctx->user1map_addr	= 0x44;
		ctx->user2map_addr	= 0x60;
		ctx->user3map_addr	= 0x48;
		ctx->reservedmap_addr	= 0x4c;
		ctx->hdmimap_addr	= 0x68;
		ctx->rksvmap_addr	= 0x64;
		ctx->edidmap_addr	= 0x6c;
	} else
		return -EINVAL;

	ctx->brightness = 128;
	ctx->contrast = 0x80;
	ctx->hue = 0;
	ctx->saturation = 0x80;

	vc8x0_ad7441_set_edid(ctx, &edid_data[0], sizeof(edid_data));
	vc8x0_ad7441_set_input(ctx, VID_INPUT_DVI);

	ctx->vga_timing_mode = vc8x0_vga_lookup(2, 640, 480, 1);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	return 0;
}

int vc8x0_ad7441_enable(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s(channel=%d)\n", __func__, ctx->dma_channel->nr);

	vc8x0_ad7441_sw_reset(ctx);

	if (ctx->dma_channel->nr == 2) {
		vc8x0_ad7441_HDMI_1920x1080px60(ctx, 0);
	} else
	if (ctx->dma_channel->nr == 1){
		vc8x0_ad7441_HDMI_1920x1080px60(ctx, 0);
	}

	return 0;
}

int vc8x0_ad7441_disable(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);
	return 0;
}

static int vc8x0_ad7441_CVBS_720x480ix60(struct vc8x0_ad7441_context *ctx, int n)
{
	u8 reg;
	dprintk(1, "%s(%d)\n", __func__, n);

	if (n == 1)
		reg = 0x00; /* CVBS on AIN1 (Green) */
	else
	if (n == 2)
		reg = 0x02; /* CVBS on AIN3 (Red) */
	else
	if (n == 3)
		reg = 0x01; /* CVBS on AIN2 (Blue) */
	else
		BUG();

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x01);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x17, 0x41);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x31, 0x10);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x34, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x35, 0x22);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, 0x50);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x00);

	return 0;
}

static int vc8x0_ad7441_SVIDEO_720x480ix60(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* Y on AIN1 (Green) */
	/* C on AIN3 (Red) */

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x03);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x17, 0x41);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x31, 0x10);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x34, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x35, 0x22);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, 0x50);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x00);

	return 0;
}

static int vc8x0_ad7441_UXGA_1600x1200x60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 488, eav = 2094;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2, /* DMT */
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines--;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d, bl = %d)\n", __func__, ushtotal, frll, lines,
		ctx->vga_timing_mode->bl);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_WXGA_1440x900x60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 381, eav = 1825;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		1, /* GTF */
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines++;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);


	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_WSXGA_1680x1050x60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 357, eav = 1761;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGAHD_1920x1080x60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 489, eav = 2413;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		1, /* GTF */
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines--;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d, bl = %d)\n", __func__, ushtotal, frll, lines, ctx->vga_timing_mode->bl);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

/* Supported VGA resolutions */
static int vc8x0_ad7441_VGA_640x480ix60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 160, eav = 806;
	u8 reg;

	sav = 137;
	eav = 783;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines--;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGA_800x600px60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 234, eav = 1040;
	u8 reg;

	sav = 210;
	eav = 1016;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines = 629;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGA_1024x768px60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 290, eav = 1320;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines++;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);


	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGA_1280x960px60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	/* 1280B@60 */

	u32 ushtotal, frll, lines, sav = 420, eav = 1706;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines++;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);


	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGA_1280x720px60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 317, eav = 1603;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines = 747;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);


	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGA_1280x1024px60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	/* 1280_@60 */
	u32 ushtotal, frll, lines, sav = 354, eav = 1640;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		2,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines++;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d)\n", __func__, ushtotal, frll, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);


	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_VGA_1400x1050px60(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	u32 ushtotal, frll, lines, sav = 382, eav = 1786;
	u8 reg;

	ctx->vga_timing_mode = vc8x0_vga_lookup(
		1,
		fmt->width,
		fmt->height,
		fmt->flags == FORMAT_INTERLACED ? 1 : 0);
	if (ctx->vga_timing_mode)
		vc8x0_vga_dump(ctx->vga_timing_mode);

	if (!ctx->vga_timing_mode) {
		printk(KERN_ERR "%s() unable to locate fmt->id %d [%s] in the timing table\n",
			__func__, fmt->id, fmt->name);
		return -ENODEV;
	}

	ushtotal = ctx->vga_timing_mode->total_pixels;
	frll = ctx->vga_timing_mode->bl >> 3;
	lines = ctx->vga_timing_mode->total_lines;
	lines++;

	dprintk(1, "%s(ushtotal = %d, frll = %d, lines = %d(%x))\n", __func__, ushtotal, frll, lines, lines);

	/* Basic input settings */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	/* Auto Mode */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x5c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x02);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xf3);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa3);

	/* ADV7441 HW RevJ Page 190 Section 9.15 */

	/* PLL_DIV_MAN_EN */
	/* PLL_DIV_RATIO */
	reg = 0xe0 | ((ushtotal & 0xfff) >> 8);
	printk(KERN_ERR "87 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x87, reg);

	reg = ushtotal & 0xff;
	printk(KERN_ERR "88 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x88, reg);

	/* FR_LL */
	reg = 0x70 | ((frll & 0x7ff) >> 8);
	printk(KERN_ERR "8f = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x8f, reg);

	reg = frll & 0xff;
	printk(KERN_ERR "90 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x90, reg);

	/* LCOUNT_MAX */
	reg = (lines & 0xfff) >> 4;
	printk(KERN_ERR "ab = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xab, reg);
	reg = (lines & 0x00f) << 4;
	printk(KERN_ERR "ac = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xac, reg);

	/* INTERLACED */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x91, 0x10);

	/* SAV / EAV */
	reg = sav >> 4;
	printk(KERN_ERR "a2 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa2, reg);
	reg = (eav >> 8) | (sav << 4);
	printk(KERN_ERR "a3 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa3, reg);
	reg = (eav & 0xff);
	printk(KERN_ERR "a4 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa4, reg);

	/* VBI */
	reg = (lines + 1) >> 4;
	printk(KERN_ERR "a5 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa5, reg);
	reg = ((lines + 1) << 4) | (((lines + 1) - fmt->height) & 0xfff) >> 8;
	printk(KERN_ERR "a6 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa6, 0xd0);
	reg = ((lines + 1) - fmt->height) & 0xff;
	printk(KERN_ERR "a7 = %02x\n", reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xa7, reg);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x00);

	return 0;
}

static int vc8x0_ad7441_YPRPB_720x480ix60(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* 0820 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x15);

	return 0;
}

static int vc8x0_ad7441_YPRPB_720x576px25(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* 0820 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x01); // 1
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x05); // 1
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x15);

	return 0;
}

static int vc8x0_ad7441_YPRPB_720x576ix50(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* 0820 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x00); // 1
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0d); // 1
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x15);

	return 0;
}

static int vc8x0_ad7441_YPRPB_1280x720p(struct vc8x0_ad7441_context *ctx, int rateHz)
{
	u8 reg;
	dprintk(1, "%s(%d)\n", __func__, rateHz);

	switch (rateHz) {
	case 50: reg = 0x2a; break;
	case 30: reg = 0x4a; break;
	case 25: reg = 0x6a; break;
	case 24: reg = 0x8a; break;
	default:
	case 60: reg = 0x0a;
	}

	/* 0820 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x15);

	return 0;
}

static int vc8x0_ad7441_YPRPB_1920x1080px30(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* 0820 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x15);

	return 0;
}

static int vc8x0_ad7441_YPRPB_1920x1080i(struct vc8x0_ad7441_context *ctx, int rateHz)
{
	u8 reg;
	dprintk(1, "%s(%d)\n", __func__, rateHz);

	switch (rateHz) {
	case 50: reg = 0x2c; break;
	case 30: reg = 0x4c; break;
	case 25: reg = 0x6c; break;
	case 24: reg = 0x8c; break;
	default:
	case 60: reg = 0x0c;
	}

	/* 0820 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x69, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x00, 0x00);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc3, 0x31);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc4, 0xc2);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3a, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x00);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x01);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, reg);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x15);

	return 0;
}

static int vc8x0_ad7441_HDMI_720x480ix60(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x37);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0a, 0xff);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x85, 0x19);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa0 | 1); /* HDMI_MODE = 1 FREE RUN */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc8, 0x08);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x95);

	/* VRX_USER_2_MAP_ADDR */
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf0, 0x10);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf1, 0x0f);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf4, 0x20);

	/* VRX_HDMI_MAP_ADDR */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x41, 0x51); /* Pixel Repetition override and 2X */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x15, 0xec);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1c, 0x4b);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1d, 0x04);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3c, 0x82);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x5a, 0x00);

	return 0;
}

static int vc8x0_ad7441_HDMI_720x576px25(struct vc8x0_ad7441_context *ctx)
{
	/* HDMI res = 625P */
	dprintk(1, "%s()\n", __func__);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x05);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x05);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x37);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0a, 0xff);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x85, 0x19);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa0 | 1); /* HDMI_MODE = 1 FREE RUN */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc8, 0x08);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x95);

	/* VRX_USER_2_MAP_ADDR */
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf0, 0x10);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf1, 0x0f);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf4, 0x20);

	/* VRX_HDMI_MAP_ADDR */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x15, 0xec);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1c, 0x4b);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1d, 0x04);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3c, 0x82);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x5a, 0x00);

	return 0;
}

static int vc8x0_ad7441_HDMI_720x576ix50(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0d);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x37);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0a, 0xff);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x85, 0x19);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa0 | 1); /* HDMI_MODE = 1 FREE RUN */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc8, 0x08);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x95);

	/* VRX_USER_2_MAP_ADDR */
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf0, 0x10);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf1, 0x0f);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf4, 0x20);

	/* VRX_HDMI_MAP_ADDR */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x41, 0x51); /* Pixel Repetition override and 2X */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x15, 0xec);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1c, 0x4b);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1d, 0x04);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3c, 0x82);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x5a, 0x00);

	return 0;
}

static int vc8x0_ad7441_HDMI_1280x720px60(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* HDMI PORT A 1280x720p 60Hz 20bit 422 */
	/* VC8x0 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x05);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x37);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0a, 0xff);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x85, 0x19);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa0 | 1); /* HDMI_MODE = 1 FREE RUN */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc8, 0x08);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x95);

	/* VRX_USER_2_MAP_ADDR */
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf0, 0x10);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf1, 0x0f);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf4, 0x20);

	/* VRX_HDMI_MAP_ADDR */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x15, 0xec);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1c, 0x4b);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1d, 0x04);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3c, 0x82);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x5a, 0x00);

	return 0;
}

static int vc8x0_ad7441_HDMI_1920x1080ix60(struct vc8x0_ad7441_context *ctx)
{
	dprintk(1, "%s()\n", __func__);

	/* HDMI PORT A 1920x1080i 60Hz 20bit 422 */
	/* VC8x0 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x05);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0c); // c, e
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x37);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0a, 0xff);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x85, 0x19);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa0 | 1); /* HDMI_MODE = 1 FREE RUN */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc8, 0x08);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x95);

	/* VRX_USER_2_MAP_ADDR */
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf0, 0x10);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf1, 0x0f);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf4, 0x20);

	/* VRX_HDMI_MAP_ADDR */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x15, 0xec);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1c, 0x4b);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1d, 0x04);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3c, 0x82);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x5a, 0x00);

	return 0;
}

static int vc8x0_ad7441_HDMI_1920x1080px60(struct vc8x0_ad7441_context *ctx, int tdmsfix)
{
	dprintk(1, "%s(%d)\n", __func__, tdmsfix);

	/* 1920x1080p */
	/* VC8x0 */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x03, 0x85);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x04, 0x1c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x05, 0x05);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x06, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0c, 0x3c);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x1d, 0x40);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x37, 0xa1);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x3c, 0xa8);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0a, 0xff);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x47, 0x0a);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x68, 0xf0);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x6b, 0xe7);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x7b, 0x0f);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x85, 0x19);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x86, 0x0b);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xba, 0xa0 | 1); /* HDMI_MODE = 1 FREE RUN */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc9, 0x04);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xc8, 0x08);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf3, 0x07);
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xf4, 0x95);

	/* VRX_USER_2_MAP_ADDR */
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf0, 0x30);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf1, 0x0f);
	vc8x0_ad7441_i2c_user2map_write8(ctx, 0xf4, 0xa0);

	/* VRX_HDMI_MAP_ADDR */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x41, 0x40); /* Pixel Repetition Not set (1x) */
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x15, 0xec);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1c, 0x4b);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x1d, 0x04);
	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3c, 0x82);

	if (tdmsfix)
		vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x47, 0x00);
	else
		vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x47, 0x05);

	vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x5a, 0x01);

	return 0;
}


int vc8x0_ad7441_get_format(struct vc8x0_ad7441_context *ctx, struct vc8x0_format **fmt)
{
	*fmt = ctx->fmt;
	return 0;
}

static int vc8x0_ad7441_set_format_DVI(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	int ret = 0;

	dprintk(1, "%s(%p, %d) [%s]\n", __func__, ctx, fmt->id, fmt->name);

	switch (fmt->id) {
	case FORMAT_720x576p25:
		vc8x0_ad7441_HDMI_720x576px25(ctx);
		break;
	case FORMAT_720x576i50:
		vc8x0_ad7441_HDMI_720x576ix50(ctx);
		break;
	case FORMAT_720x480i59:
	case FORMAT_720x480i60:
		vc8x0_ad7441_HDMI_720x480ix60(ctx);
		break;
	case FORMAT_1280x720p23:
	case FORMAT_1280x720p24:
	case FORMAT_1280x720p25:
	case FORMAT_1280x720p29:
	case FORMAT_1280x720p30:
	case FORMAT_1280x720p50:
	case FORMAT_1280x720p59:
	case FORMAT_1280x720p60:
		vc8x0_ad7441_HDMI_1280x720px60(ctx);
		break;
	case FORMAT_1920x1080i50:
	case FORMAT_1920x1080i59:
	case FORMAT_1920x1080i60:
		vc8x0_ad7441_HDMI_1920x1080ix60(ctx);
		break;
	case FORMAT_1920x1080p23:
	case FORMAT_1920x1080p24:
	case FORMAT_1920x1080p25:
	case FORMAT_1920x1080p29:
	case FORMAT_1920x1080p30:
	case FORMAT_1920x1080p50:
		vc8x0_ad7441_HDMI_1920x1080px60(ctx, 0);
		break;
	case FORMAT_1920x1080p60:
		vc8x0_ad7441_HDMI_1920x1080px60(ctx, 1);
		break;
	default:
		printk(KERN_ERR "%s() huh?\n", __func__);
		ret = -EINVAL;
	}

	return ret;
}

static int vc8x0_ad7441_set_format_YPRPB(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	int ret = 0;

	dprintk(1, "%s(%p, %d) [%s]\n", __func__, ctx, fmt->id, fmt->name);

	switch (fmt->id) {
	case FORMAT_720x480i59:
	case FORMAT_720x480i60:
		vc8x0_ad7441_YPRPB_720x480ix60(ctx);
		break;
	case FORMAT_720x576p25:
		vc8x0_ad7441_YPRPB_720x576px25(ctx);
		break;
	case FORMAT_720x576i50:
		vc8x0_ad7441_YPRPB_720x576ix50(ctx);
		break;
	case FORMAT_1280x720p23:
	case FORMAT_1280x720p24:
		vc8x0_ad7441_YPRPB_1280x720p(ctx, 24);
		break;
	case FORMAT_1280x720p25:
		vc8x0_ad7441_YPRPB_1280x720p(ctx, 25);
		break;
	case FORMAT_1280x720p29:
	case FORMAT_1280x720p30:
		vc8x0_ad7441_YPRPB_1280x720p(ctx, 30);
		break;
	case FORMAT_1280x720p50:
		vc8x0_ad7441_YPRPB_1280x720p(ctx, 50);
		break;
	case FORMAT_1280x720p59:
	case FORMAT_1280x720p60:
		vc8x0_ad7441_YPRPB_1280x720p(ctx, 60);
		break;
	case FORMAT_1920x1080p30:
		vc8x0_ad7441_YPRPB_1920x1080px30(ctx);
		break;
	case FORMAT_1920x1080i50:
		vc8x0_ad7441_YPRPB_1920x1080i(ctx, 50);
		break;
	case FORMAT_1920x1080i59:
	case FORMAT_1920x1080i60:
		vc8x0_ad7441_YPRPB_1920x1080i(ctx, 60);
		break;
	default:
		printk(KERN_ERR "%s(%p, %d) huh?\n", __func__, ctx, fmt->id);
		ret = -EINVAL;
	}

	return ret;
}

static int vc8x0_ad7441_set_format_COMPOSITE(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt, int inputnr)
{
	int ret = 0;

	dprintk(1, "%s(%p, %d, %d) [%s]\n", __func__, ctx, fmt->id, inputnr, fmt->name);

	switch (fmt->id) {
	case FORMAT_720x576p25:
	case FORMAT_720x576i50:
		vc8x0_ad7441_CVBS_720x480ix60(ctx, inputnr);	
		ctx->dma_channel->vbi_enabled = 0;
		break;
	case FORMAT_720x480i59:
	case FORMAT_720x480i60:
		vc8x0_ad7441_CVBS_720x480ix60(ctx, inputnr);
		ctx->dma_channel->vbi_enabled = 1;
		break;
	default:
		printk(KERN_ERR "%s(%p, %d, %d) huh?\n", __func__, ctx, fmt->id, inputnr);
		ret = -EINVAL;
	}

	return ret;
}

static int vc8x0_ad7441_set_format_SVIDEO(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	int ret = 0;

	dprintk(1, "%s(%p, %d) [%s]\n", __func__, ctx, fmt->id, fmt->name);

	switch (fmt->id) {
	case FORMAT_720x576p25:
	case FORMAT_720x576i50:
		vc8x0_ad7441_SVIDEO_720x480ix60(ctx);
		ctx->dma_channel->vbi_enabled = 0;
		break;
	case FORMAT_720x480i59:
	case FORMAT_720x480i60:
		vc8x0_ad7441_SVIDEO_720x480ix60(ctx);
		ctx->dma_channel->vbi_enabled = 1;
		break;
	default:
		printk(KERN_ERR "%s(%p, %d) huh?\n", __func__, ctx, fmt->id);
		ret = -EINVAL;
	}

	return ret;
}


static int vc8x0_ad7441_set_format_VGA(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	int ret = 0;

	dprintk(1, "%s(%p, %d) [%s]\n", __func__, ctx, fmt->id, fmt->name);

	switch (fmt->id) {
	case FORMAT_640x480p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_640x480ix60(ctx, fmt);
		break;
	case FORMAT_800x600p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_800x600px60(ctx, fmt);
		break;
	case FORMAT_1024x768p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_1024x768px60(ctx, fmt);
		break;
	case FORMAT_1280x720p23:
	case FORMAT_1280x720p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_1280x720px60(ctx, fmt);
		break;
	case FORMAT_1280x960p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_1280x960px60(ctx, fmt);
		break;
	case FORMAT_1280x1024p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_1280x1024px60(ctx, fmt);
		break;
	case FORMAT_1400x1050p60:
		ctx->dma_channel->vbi_enabled = 0;
		vc8x0_ad7441_VGA_1400x1050px60(ctx, fmt);
		break;
	case FORMAT_1600x1200p60:
	case FORMAT_UXGA_1600x1200p60:
		vc8x0_ad7441_UXGA_1600x1200x60(ctx, fmt);
		ctx->dma_channel->vbi_enabled = 0;
		break;
	case FORMAT_WXGA_1440x900p60:
		vc8x0_ad7441_WXGA_1440x900x60(ctx, fmt);
		ctx->dma_channel->vbi_enabled = 0;
		break;
	case FORMAT_WSXGA_1680x1050p60:
		vc8x0_ad7441_WSXGA_1680x1050x60(ctx, fmt);
		ctx->dma_channel->vbi_enabled = 0;
		break;
	case FORMAT_VGAHD_1920x1080p60:
		vc8x0_ad7441_VGAHD_1920x1080x60(ctx, fmt);
		ctx->dma_channel->vbi_enabled = 0;
		break;
	default:
		printk(KERN_ERR "%s(%p, %d) huh?\n", __func__, ctx, fmt->id);
		ret = -EINVAL;
	}

	return ret;
}


int vc8x0_ad7441_set_format(struct vc8x0_ad7441_context *ctx, struct vc8x0_format *fmt)
{
	int ret = -ENODEV;
	dprintk(1, "%s(%p, %d)\n", __func__, ctx, fmt->id);

	vc8x0_ad7441_sw_reset(ctx);

	switch (ctx->video_input) {
	case VID_INPUT_DVI:
		ret = vc8x0_ad7441_set_format_DVI(ctx, fmt);
		break;
	case VID_INPUT_YPRPB:
		ret = vc8x0_ad7441_set_format_YPRPB(ctx, fmt);
		break;
	case VID_INPUT_VGA:
		ret = vc8x0_ad7441_set_format_VGA(ctx, fmt);
		break;
	case VID_INPUT_COMPOSITE1:
		ret = vc8x0_ad7441_set_format_COMPOSITE(ctx, fmt, 1);
		break;
	case VID_INPUT_COMPOSITE2:
		ret = vc8x0_ad7441_set_format_COMPOSITE(ctx, fmt, 2);
		break;
	case VID_INPUT_COMPOSITE3:
		ret = vc8x0_ad7441_set_format_COMPOSITE(ctx, fmt, 3);
		break;
	case VID_INPUT_SVIDEO:
		ret = vc8x0_ad7441_set_format_SVIDEO(ctx, fmt);
		break;
	default:
		break;
	}

	if (ret == 0)
		ctx->fmt = fmt;
	else
		ctx->fmt = 0;

	/* Free run */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0xbf, 0x12);

	return ret;	
}

int vc8x0_ad7441_set_input(struct vc8x0_ad7441_context *ctx, enum video_input_type input)
{
	int ret = 0;
	dprintk(1, "%s(%p, %d [%s])\n", __func__, ctx,
		input,
		input == VID_INPUT_DVI ? "DVI" :
		input == VID_INPUT_YPRPB ? "YPRPB" :
		input == VID_INPUT_VGA ? "VGA" :
		input == VID_INPUT_SVIDEO ? "SVIDEO" :
		input == VID_INPUT_COMPOSITE1 ? "COMPOSITE1" :
		input == VID_INPUT_COMPOSITE2 ? "COMPOSITE2" :
		input == VID_INPUT_COMPOSITE3 ? "COMPOSITE3" :
		"UNDEFINED"
		);
	ctx->video_input = input;

	return ret;
}

int vc8x0_ad7441_sw_reset(struct vc8x0_ad7441_context *ctx)
{
	/* Put the chip into s/w reset */
	vc8x0_ad7441_i2c_usermap_write8(ctx, 0x0F, 0x80);
	msleep(20);

	return 0;
}

/* Controls */
int vc8x0_ad7441_set_brightness(struct vc8x0_ad7441_context *ctx, u32 v)
{
	dprintk(1, "%s(0x%x)\n", __func__, v);
	ctx->brightness = v;

	/* Enable the Video Adjustment and CSC block */
	vc8x0_ad7441_i2c_user1map_setbit(ctx, 0x9e, 0x80);
	vc8x0_ad7441_i2c_usermap_setbit(ctx, 0x69, 0x18);

	vc8x0_ad7441_i2c_user1map_write8(ctx, 0x9c, v - 128);
	return 0;
}

int vc8x0_ad7441_get_brightness(struct vc8x0_ad7441_context *ctx, u32 *v)
{
	*v = ctx->brightness;
	dprintk(1, "%s(0x%x)\n", __func__, *v);
	return 0;
}

int vc8x0_ad7441_set_contrast(struct vc8x0_ad7441_context *ctx, u32 v)
{
	dprintk(1, "%s(0x%x)\n", __func__, v);
	ctx->contrast = v;

	/* Enable the Video Adjustment and CSC block */
	vc8x0_ad7441_i2c_user1map_setbit(ctx, 0x9e, 0x80);
	vc8x0_ad7441_i2c_usermap_setbit(ctx, 0x69, 0x18);

	vc8x0_ad7441_i2c_user1map_write8(ctx, 0x9a, v);
	return 0;
}

int vc8x0_ad7441_get_contrast(struct vc8x0_ad7441_context *ctx, u32 *v)
{
	*v = ctx->contrast;
	dprintk(1, "%s(0x%x)\n", __func__, *v);
	return 0;
}

int vc8x0_ad7441_set_hue(struct vc8x0_ad7441_context *ctx, u32 v)
{
	dprintk(1, "%s(0x%x)\n", __func__, v);
	ctx->hue = v;

	/* Enable the Video Adjustment and CSC block */
	vc8x0_ad7441_i2c_user1map_setbit(ctx, 0x9e, 0x80);
	vc8x0_ad7441_i2c_usermap_setbit(ctx, 0x69, 0x18);

	vc8x0_ad7441_i2c_user1map_write8(ctx, 0x9d, v);
	return 0;
}

int vc8x0_ad7441_get_hue(struct vc8x0_ad7441_context *ctx, u32 *v)
{
	*v = ctx->hue;
	dprintk(1, "%s(0x%x)\n", __func__, *v);
	return 0;
}

int vc8x0_ad7441_set_saturation(struct vc8x0_ad7441_context *ctx, u32 v)
{
	dprintk(1, "%s(0x%x)\n", __func__, v);
	ctx->saturation = v;

	/* Enable the Video Adjustment and CSC block */
	vc8x0_ad7441_i2c_user1map_setbit(ctx, 0x9e, 0x80);
	vc8x0_ad7441_i2c_usermap_setbit(ctx, 0x69, 0x18);

	vc8x0_ad7441_i2c_user1map_write8(ctx, 0x9b, v);
	return 0;
}

int vc8x0_ad7441_get_saturation(struct vc8x0_ad7441_context *ctx, u32 *v)
{
	*v = ctx->saturation;
	dprintk(1, "%s(0x%x)\n", __func__, *v);
	return 0;
}

int vc8x0_ad7441_set_edid(struct vc8x0_ad7441_context *ctx, u8 *data, int len)
{
	int i;
	dprintk(1, "%s(len=%d)\n", __func__, len);

	if (len != 256) {
		printk(KERN_ERR "%s() edid len %d is invalid\n", __func__, len);
		return -EINVAL;
	}

	for (i = 0; i < 256; i++)
		vc8x0_ad7441_i2c_edidmap_write8(ctx, i, *(data + i));

	vc8x0_ad7441_i2c_rksvmap_write8(ctx, 0x70, 0x02);
	vc8x0_ad7441_i2c_rksvmap_write8(ctx, 0x71, 0x03);
	vc8x0_ad7441_i2c_rksvmap_write8(ctx, 0x72, 0x80);
	vc8x0_ad7441_i2c_rksvmap_write8(ctx, 0x73, 0x06);

	msleep(20);
	return 0;
}

void vc8x0_ad7441_procfs_detection_show(struct seq_file *m, struct vc8x0_ad7441_context *ctx)
{
	struct detection_t *d = &ctx->detection;

	seq_printf(m, "Detected settings:\n");
	seq_printf(m, " ->locked = %d\n", d->locked);
	seq_printf(m, " ->interlaced = %d\n", d->interlaced);
	seq_printf(m, " ->sync_mask = %d\n", d->sync_mask);
	seq_printf(m, " ->sync_polarity = 0x%02x\n", d->sync_polarity);
	seq_printf(m, " ->lcf = %d\n", d->lcf);
	seq_printf(m, " ->lcvs = %d\n", d->lcvs);
	seq_printf(m, " ->fcl28 = %d\n", d->fcl28);
	seq_printf(m, " ->bl = %d\n", d->bl);
}

int vc8x0_ad7441_procfs_show(struct seq_file *m, struct vc8x0_ad7441_context *ctx)
{
	int i, c = 0, rate = 0;
	unsigned char buf[0x100];
	seq_printf(m, "AD7441:\n");

	for (i = 0; i < sizeof(buf); i++)
		vc8x0_ad7441_i2c_hdmimap_read8(ctx, i, &buf[i]);

	seq_printf(m, "HDMI 00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f\n");
	for (i = 0; i < sizeof(buf); i++) {
		if (c == 0)
			seq_printf(m, " %02x:", i);

		seq_printf(m, " %02x", buf[i]);

		if (++c == 16) {
			seq_printf(m, "\n");
			c = 0;
		}
	}

	seq_printf(m, "->Audio PLL Locked = %d\n", (buf[0x04] >> 0) & 0x01);
	seq_printf(m, "->Video PLL Locked = %d\n", (buf[0x04] >> 1) & 0x01);
	seq_printf(m, "->TMDS Port B Active = %d\n", (buf[0x04] >> 2) & 0x01);
	seq_printf(m, "->TMDS Port A Active = %d\n", (buf[0x04] >> 3) & 0x01);
	seq_printf(m, "->AV Muted = %d\n", (buf[0x04] >> 6) & 0x01);
	seq_printf(m, "->Video Width Detected = %d\n", ((buf[0x07] << 8) | buf[0x08]) & 0xfff);
	seq_printf(m, "->Video Height Detected = %d\n", ((buf[0x09] << 8) | buf[0x0a]) & 0xfff);

	for (i = 0; i < sizeof(buf); i++)
		vc8x0_ad7441_i2c_usermap_read8(ctx, i, &buf[i]);

	seq_printf(m, "USR  00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f\n");
	c = 0;
	for (i = 0; i < sizeof(buf); i++) {
		if (c == 0)
			seq_printf(m, " %02x:", i);

		seq_printf(m, " %02x", buf[i]);

		if (++c == 16) {
			seq_printf(m, "\n");
			c = 0;
		}
	}
#if 0
	for (i = 0; i < sizeof(buf); i++)
		vc8x0_ad7441_i2c_edidmap_read8(ctx, i, &buf[i]);

	seq_printf(m, "EDID 00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f\n");
	c = 0;
	for (i = 0; i < sizeof(buf); i++) {
		if (c == 0)
			seq_printf(m, " %02x:", i);

		seq_printf(m, " %02x", buf[i]);

		if (++c == 16) {
			seq_printf(m, "\n");
			c = 0;
		}
	}
	if (memcmp(edid_data, buf, sizeof(edid_data)) != 0)
		seq_printf(m, "Edid corrupt?\n");
#endif
	memset(buf, 0, sizeof(buf));
	for (i = 0x60; i < 0x70; i++)
		vc8x0_ad7441_i2c_user1map_read8(ctx, i, &buf[i]);

	seq_printf(m, "USR1 00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f\n");
	c = 0;
	for (i = 0; i < sizeof(buf); i++) {
		if (c == 0)
			seq_printf(m, " %02x:", i);

		seq_printf(m, " %02x", buf[i]);

		if (++c == 16) {
			seq_printf(m, "\n");
			c = 0;
		}
	}

	switch (buf[6] & 0x0f) {
	case 0x0a:
		seq_printf(m, "->Resolution Requested = 1280x720p\n");
		break;
	case 0x0c:
	case 0x0e:
		seq_printf(m, "->Resolution Requested = 1920x1080i\n");
		break;
	}

	switch ((buf[6] & 0xe0) >> 5) {
	case 0x00:
		rate = 60;
		break;
	case 0x01:
		rate = 50;
		break;
	case 0x02:
		rate = 30;
		break;
	case 0x03:
		rate = 25;
		break;
	case 0x04:
		rate = 24;
		break;
	}
	seq_printf(m, "->Rate Requested = %dHz\n", rate);
	seq_printf(m, "->Format Requested = %d [%s]\n",
		ctx->fmt ? ctx->fmt->id : -1,
		ctx->fmt ? ctx->fmt->name : "undefined");

	return 0;
}

void vc8x0_ad7441_audio_reset_pll(struct vc8x0_ad7441_context *ctx)
{
	/* Reset audio PLL */
	vc8x0_ad7441_i2c_hdmimap_setbit(ctx, 0x5a, 0x03);
}

void vc8x0_ad7441_audio_configure_pll(struct vc8x0_ad7441_context *ctx)
{
	u8 v, t[3];
	u32 n, cts, tdms;
	u32 tmp, tmp2, tmp3;
	u32 xtal = 28636360;

	/* GET N */
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5b, &v);
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5d, &t[0]);
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5e, &t[1]);
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5f, &t[2]);
	n = ((t[0] & 0x0f) << 16) | (t[1] << 8) | t[2];

	/* GET CTS */
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5b, &t[0]);
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5c, &t[1]);
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x5d, &t[2]);
	cts = (t[0] << 12) | (t[1] << 4) | ((t[2] & 0xf0) >> 4);
	
	/* GET TDMS Frequency */
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x06, &v);
	tmp = xtal / 10;
	tmp2 = 27000000 / 10;
	tmp3 = tmp * v;
	tdms = tmp3 / tmp2;

	/* Update the Audio PLL */
	if (cts) {
		tmp = tdms * (n / cts);
		tmp = (tmp * 1000) >> 7;
		if (tmp >= 150) {
			dprintk(1, "%s() Setting A\n", __func__);
			vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3d, 0x80);
		} else {
			dprintk(1, "%s() Setting B\n", __func__);
			vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3d, 0x40);
		}
	} else {
		dprintk(1, "%s() ELSE CTS\n", __func__);
		vc8x0_ad7441_i2c_hdmimap_write8(ctx, 0x3d, 0x40);
	}

	dprintk(1, "%s() Audio Frequency: %d\n", __func__, tmp);
}

int vc8x0_ad7441_keep_alive(struct vc8x0_ad7441_context *ctx)
{
	u8 val, clr = 0;

	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x1a, &val);

	if (val & 0x10) {
		/* NEW_CTS */
		vc8x0_ad7441_audio_reset_pll(ctx);
		clr |= 0x10;
	}

	if (val & 0x08) {
		/* NEW_N */
		vc8x0_ad7441_audio_configure_pll(ctx);
		clr |= 0x08;
	}

	if (clr)
		vc8x0_ad7441_i2c_user1map_write8(ctx, 0x6f, clr);

	vc8x0_ad7441_i2c_user1map_read8(ctx, 0x64, &val);
	if (val & 0x02) {
		dprintk(1, "%s() 64\n", __func__);
	}

	return 0;
}

static int vc8x0_ad7441_is_hdmi_locked(struct vc8x0_ad7441_context *ctx)
{
	u8 reg4, reg7;

	/* Read the Audio/Video PLL's */
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x04, &reg4);
	dprintk(2, "%s(nr=%d) 0x04 = %02x\n", __func__, ctx->dma_channel->nr, reg4); /* Add this */

	/* Is Video PLL is locked, is TMDS_PORT_A is active */
	if ((reg4 & 0x0a) != 0x0a)
		return 0;

	/* Is the vertical filter locked? */
	vc8x0_ad7441_i2c_hdmimap_read8(ctx, 0x07, &reg7);
	dprintk(2, "%s(nr=%d) 0x07 = %02x\n", __func__, ctx->dma_channel->nr, reg7); /* Add this */
	if ((reg7 & 0x40) == 0x00)
		return 0;

	/* Is the horizontal filter locked? */
	if ((reg7 & 0x10) == 0x00)
		return 0;

	/* Locked */
	return 1;
}

static int vc8x0_ad7441_is_YPRPB_locked(struct vc8x0_ad7441_context *ctx)
{
	u8 reg12;

	/* Is the TLLC_PLL_LOCK bit set */
	vc8x0_ad7441_i2c_usermap_read8(ctx, 0x12, &reg12);
	if ((reg12 & 0x80) != 0x80)
		return 0;

	/* Is the CP free-running? If set then no signal */
	if ((reg12 & 0x40) == 0x40)
		return 0;

	/* Locked */
	return 1;
}

static int vc8x0_ad7441_is_composite_locked(struct vc8x0_ad7441_context *ctx)
{
	u8 reg10;

	/* Is the IN_LOCK bit set */
	vc8x0_ad7441_i2c_usermap_read8(ctx, 0x10, &reg10);
	if ((reg10 & 0x01) == 0x00)
		return 0;

	/* Locked */
	return 1;
}

/* Driver Lock detect and format recognition works in the following way.
 * If we're using a component input then if the CP processor detects
 * we're not locked or in free-running mode, request colorbar generation.
 *
 * If we're using a component input then if the CP processor detects
 * we're correctly locked, attempt to detect the format based on the STDI
 * setting. If the STDI setting match the user requested format or rate, disable
 * colorbars. If the STDU setting do not match the user requested format or
 * rate, request colorbars.
 *
 * If we're using a hdmi input then if the CP processor detects
 * we're correctly locked, attempt to detect the format based on the STDI
 * setting. If the STDI setting match the user requested format or rate, disable
 * colorbars. If the STDU setting do not match the user requested format or
 * rate, request colorbars.
 */
/* Return 0 on not locked, 1 on locked */
int vc8x0_ad7441_is_locked(struct vc8x0_ad7441_context *ctx)
{
	struct vc8x0_dma_channel *channel = ctx->dma_channel;
	struct vc8x0_dev *dev = channel->dev;
	int ret;

	if (!time_after(jiffies, (long)ctx->last_locked + msecs_to_jiffies(500)))
	{
		/* Skip the detect, because it hasn't been 500ms since the last attempt */
		return 1;
	}
	ctx->last_locked = jiffies;

	switch (vc8x0_boards[dev->board].input[channel->video_inputnr].type) {
	case VC8X0_VMUX_DVI:
		ret = vc8x0_ad7441_is_hdmi_locked(ctx);
		break;
	case VC8X0_VMUX_YPRPB:
	case VC8X0_VMUX_VGA:
		ret = vc8x0_ad7441_is_YPRPB_locked(ctx);
		break;
	case VC8X0_VMUX_COMPOSITE1:
	case VC8X0_VMUX_COMPOSITE2:
	case VC8X0_VMUX_COMPOSITE3:
	case VC8X0_VMUX_SVIDEO:
		ret = vc8x0_ad7441_is_composite_locked(ctx);
		break;
	default:
		ret = 0;
	}

	if (ret == 0) {
		/* We didn't detect the video lock correctly, throw the colorbars up. */
		vc8x0_channel_loss_of_sync(ctx->dma_channel, 1);
	} else {
		vc8x0_ad7441_detect(ctx);
	}

	return ret;
}

/* return 0 on format not detected or 1 on detected */
static int vc8x0_ad7441_hdmi_detect(struct vc8x0_ad7441_context *ctx)
{
	struct detection_t *d = &ctx->detection;
	struct vc8x0_dev *dev = ctx->dma_channel->dev;
	struct vc8x0_dma_channel *channel = ctx->dma_channel;
	struct vc8x0_format *f;
	struct vga_size_t *size;
	struct vga_rate_t *rate;
	u8 stat_10[4], stdi_b1[5], fcl[2];
	int i, colorbars_reqd = 1;

	for (i = 0; i < 5; i++)
		vc8x0_ad7441_i2c_usermap_read8(ctx, 0xb1 + i, &stdi_b1[i]);

	/* See the ADV7441 HW manual section 9.10 rev J page 177 */
	dprintk(2, "%s(channel=%d) STDI b1-b5: %02x %02x %02x %02x %02x\n", __func__, ctx->dma_channel->nr,
		stdi_b1[0], stdi_b1[1], stdi_b1[2], stdi_b1[3], stdi_b1[4]);

	/* if STDI_DVALID */
	if (stdi_b1[0] & 0x80) {

		/* Format is detected */
		d->valid = 1;

		/* if STDI_INTLCD */
		if (stdi_b1[0] & 0x40)
			d->interlaced = 1;
		else
			d->interlaced = 0;

		/* Block length */
		d->bl  = ((stdi_b1[0] & 0x3f) << 8) | stdi_b1[1];

		/* Line count in field */
		d->lcf = ((stdi_b1[2] & 0x07) << 8) | stdi_b1[3];

		/* Line count in vsync */
		d->lcvs = stdi_b1[2] >> 3;

		/* Read the fractional field length */
		vc8x0_ad7441_i2c_usermap_read8(ctx, 0xca, &fcl[0]);
		vc8x0_ad7441_i2c_usermap_read8(ctx, 0xcb, &fcl[1]);
		d->fcl28 = ((fcl[0] & 0x1f) << 8) | fcl[1];

		/* if SSPD_DVALID */
		if (stdi_b1[4] & 0x80) {
			d->sync_mask = stdi_b1[4] & 0x03;
			d->sync_polarity = ((stdi_b1[4] & 0x08) >> 2) | ((stdi_b1[4] & 0x20) >> 5);
		}

		d->field_rate = 0;
		if (d->interlaced)
			d->field_rate /= 2;
	}

	for (i = 0; i < 4; i++)
		vc8x0_ad7441_i2c_usermap_read8(ctx, 0x10 + i, &stat_10[i]);

	/* If CP_FREE_RUN is active then we're not locked to incoming video... */
	if (stat_10[2] & 40)
		d->locked = 0;
	else
		d->locked = 1;

	/* Lookup the params in the vga table and detect whether we're locked.
	 * If we are, try to find the matching video format and determine
	 * whether we're locked to the correct format, or some other format.
	 * If we're locked to the incorrect format then we'll warn the user.
	 */
	if (d->valid) {
		dprintk(2, "%s() lcf = %d, fcl28 = %d, bl = %d\n", __func__, d->lcf, d->fcl28, d->bl);
		size = vc8x0_vga_HdGetSize(d->lcf);
		rate = vc8x0_vga_HdGetRate(d->fcl28);
		if (size && rate) {
			dprintk(2, "%s() size = %d/%d interlaced = %d rate = %d\n",
				__func__,
				size->width, size->height[0], size->interlaced,
				rate->rate);

			do {
				if (size->interlaced) {
					/* Interlaced */
					f = vc8x0_video_find_format(ctx->dma_channel, V4L2_PIX_FMT_YUYV,
						size->width, size->height[0] * 2, FORMAT_INTERLACED, rate->rate);

					/* i59 to i60 is really close, rather than touch the rate table and risk
					 * breaking everything so late in the project, for this single format
					 * I'm going to manually override the detected format to match the
					 * callers needs.
					 */
					if ((vc8x0_boards[dev->board].input[channel->video_inputnr].type ==
						VC8X0_VMUX_YPRPB)) {
						if (f && (f->id == FORMAT_1920x1080i60)) {
							if (ctx->fmt->id == FORMAT_1920x1080i59) {
								f = ctx->fmt;
							}
						}
					}
				} else {
					if ((size->width == 720) && (size->height[0] == 483) &&
						(vc8x0_boards[dev->board].input[channel->video_inputnr].type == VC8X0_VMUX_VGA))
					{
						f = vc8x0_video_find_format(ctx->dma_channel, V4L2_PIX_FMT_YUYV,
							640, 480, FORMAT_PROGRESSIVE, 60);
					} else
					if ((size->width == 720) && (size->height[0] == 576)) {
						f = vc8x0_video_find_format(ctx->dma_channel, V4L2_PIX_FMT_YUYV,
							720, 576, FORMAT_PROGRESSIVE, 25);
					} else {
						/* Progressive */
						f = vc8x0_video_find_format(ctx->dma_channel, V4L2_PIX_FMT_YUYV,
							size->width, size->height[0], FORMAT_PROGRESSIVE, rate->rate);
					}
				}
				ctx->detected_fmt = f;
				if (f && ctx->fmt) {
					if (f != ctx->fmt) {
						dprintk(2, "%s() Detected incorrect format->id = %d [%s], need %d\n",
							__func__, f->id, f->name, ctx->fmt->id);
					} else {
						dprintk(2, "%s() Detected format->id = %d [%s] correctly.\n", __func__, f->id, f->name);
						colorbars_reqd = 0;
					}
				}

			} while (0);
		} else {
			dprintk(2, "%s() size = unknown\n", __func__);
			//colorbars_reqd = 0;
		}
	}

	if (colorbars_reqd == 1) {
		/* We didn't detect the video format correctly, throw the colorbars up. */
		vc8x0_channel_loss_of_sync(ctx->dma_channel, 1);
	} else {
		/* format is good. Remove colorbars if they were present. */
		vc8x0_channel_loss_of_sync(ctx->dma_channel, 0);
	}

	return 0;
}

/* return 0 on format not detected or 1 on detected */
static int vc8x0_ad7441_composite_detect(struct vc8x0_ad7441_context *ctx)
{
	int colorbars_reqd = 1;
	u8 ret, reg10;

	if (!ctx->fmt)
		return 0;

	/* Read the standard auto-detect bits */
	vc8x0_ad7441_i2c_usermap_read8(ctx, 0x10, &reg10);
	switch ((reg10 & 0x70) >> 4) {
	case 0x00: /* NTSC-MJ */
	case 0x01: /* NTSC-443 */
	case 0x02: /* PAL-M */
	case 0x03: /* PAL-60 */
		if (ctx->fmt->height == 480)
			colorbars_reqd = 0;
		break;
	case 0x04: /* PAL-BGHID */
	case 0x06: /* PAL-N */
		if (ctx->fmt->height == 576)
			colorbars_reqd = 0;
		break;
	case 0x05: /* SECAM */
	case 0x07: /* SECAM 525 */
	default:
		break;
	}

	if (colorbars_reqd == 1) {
		/* We didn't detect the video format correctly, throw the colorbars up. */
		vc8x0_channel_loss_of_sync(ctx->dma_channel, 1);
		ret = 0;
	} else {
		/* format is good. Remove colorbars if they were present. */
		vc8x0_channel_loss_of_sync(ctx->dma_channel, 0);
		ret = 1;
	}

	return ret;
}

/* return 0 on format not detected or 1 on detected */
static int vc8x0_ad7441_vga_detect(struct vc8x0_ad7441_context *ctx)
{
	if (!ctx->fmt)
		return 0;

	return 1;
}

/* Detect the video format and rate based on each different input type */
/* return 0 on format not detected or 1 on detected */
int vc8x0_ad7441_detect(struct vc8x0_ad7441_context *ctx)
{
	struct vc8x0_dma_channel *channel = ctx->dma_channel;
	struct vc8x0_dev *dev = channel->dev;
	int ret;
	dprintk(1, "%s()\n", __func__);

	if (!time_after(jiffies, (long)ctx->last_detected + msecs_to_jiffies(500)))
	{
		/* Skip the detect, because it hasn't been 500ms since the last attempt */
		return 1;
	}
	ctx->last_detected = jiffies;

	switch (vc8x0_boards[dev->board].input[channel->video_inputnr].type) {
	case VC8X0_VMUX_YPRPB:
	case VC8X0_VMUX_DVI:
		ret = vc8x0_ad7441_hdmi_detect(ctx);
		break;
	case VC8X0_VMUX_VGA:
		ret = vc8x0_ad7441_vga_detect(ctx);
		ret = vc8x0_ad7441_hdmi_detect(ctx);
		break;
	case VC8X0_VMUX_COMPOSITE1:
	case VC8X0_VMUX_COMPOSITE2:
	case VC8X0_VMUX_COMPOSITE3:
	case VC8X0_VMUX_SVIDEO:
		ret = vc8x0_ad7441_composite_detect(ctx);
		break;
	default:
		ret = 0;
	}

	return ret;
}

