/*
 *  Driver for the ViewCast 845e PCIe Capture Card
 *
 *  Copyright 2011/2012, Kernel Labs Inc. www.kernellabs.com.
 *   - Steven Toth <stoth@kernellabs.com>
 *
 *  Project sponsored by ViewCast Texas, US.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "vc8x0.h"

struct vc8x0_board vc8x0_boards[] = {
	[VC8X0_BOARD_UNKNOWN] = {
		.name		= "UNKNOWN/GENERIC",
		.fwtype		= FW_DEVELOPMENT,
		/* Ensure safe default for unknown boards */
		.input          = {{
			.type   = VC8X0_VMUX_YPRPB,
		}, {
			.type   = VC8X0_VMUX_DVI,
		} },
	},
	[VC8X0_BOARD_820E_DEV] = {
		.name		= "ViewCast 820e (dev)",
		.fwtype		= FW_DEVELOPMENT,
		.input          = {{
			.type   = VC8X0_VMUX_DVI,
		}, {
			.type   = VC8X0_VMUX_YPRPB,
		}, {
			.type   = VC8X0_VMUX_VGA,
		}, {
			.type   = VC8X0_VMUX_SVIDEO,
		}, {
			.type   = VC8X0_VMUX_COMPOSITE1,
		}, {
			.type   = VC8X0_VMUX_COMPOSITE2,
		}, {
			.type   = VC8X0_VMUX_COMPOSITE3,
		} },
	},
	[VC8X0_BOARD_820E] = {
		.name		= "ViewCast 820e",
		.fwtype		= FW_PRODUCTION,
		.input          = {{
			.type   = VC8X0_VMUX_DVI,
		}, {
			.type   = VC8X0_VMUX_YPRPB,
		}, {
			.type   = VC8X0_VMUX_VGA,
		}, {
			.type   = VC8X0_VMUX_SVIDEO,
		}, {
			.type   = VC8X0_VMUX_COMPOSITE1,
		}, {
			.type   = VC8X0_VMUX_COMPOSITE2,
		}, {
			.type   = VC8X0_VMUX_COMPOSITE3,
		} },
	},
	[VC8X0_BOARD_845E] = {
		.name		= "ViewCast 845e",
		.fwtype		= FW_SDI,
		.input          = {{
			.type   = VC8X0_VMUX_SDI,
		} },
	},
};
const unsigned int vc8x0_bcount = ARRAY_SIZE(vc8x0_boards);

struct vc8x0_subid vc8x0_subids[] = {
	{
		.subvendor = 0x1a39,
		.subdevice = 0x0000,
		.card      = VC8X0_BOARD_UNKNOWN,
	}, {
		.subvendor = 0x1a39,
		.subdevice = 0x0004,
		.card      = VC8X0_BOARD_820E_DEV,
	}, {
		.subvendor = 0x1576,
		.subdevice = 0x0820,
		.card      = VC8X0_BOARD_820E,
	}, {
		.subvendor = 0x1576,
		.subdevice = 0x0845,
		.card      = VC8X0_BOARD_845E,
	},
};
const unsigned int vc8x0_idcount = ARRAY_SIZE(vc8x0_subids);

void vc8x0_card_list(struct vc8x0_dev *dev)
{
	int i;

	if (0 == dev->pci->subsystem_vendor &&
	    0 == dev->pci->subsystem_device) {
		printk(KERN_INFO
			"%s: Board has no valid PCIe Subsystem ID and can't\n"
		       "%s: be autodetected. Pass card=<n> insmod option\n"
		       "%s: to workaround that. Redirect complaints to the\n"
		       "%s: vendor of the TV card.  Best regards,\n"
		       "%s:         -- tux\n",
		       dev->name, dev->name, dev->name, dev->name, dev->name);
	} else {
		printk(KERN_INFO
			"%s: Your board isn't known (yet) to the driver.\n"
		       "%s: Try to pick one of the existing card configs via\n"
		       "%s: card=<n> insmod option.  Updating to the latest\n"
		       "%s: version might help as well.\n",
		       dev->name, dev->name, dev->name, dev->name);
	}
	printk(KERN_INFO "%s: Here is a list of valid choices for the card=<n> insmod option:\n",
	       dev->name);
	for (i = 0; i < vc8x0_bcount; i++)
		printk(KERN_INFO "%s:    card=%d -> %s\n",
		       dev->name, i, vc8x0_boards[i].name);
}

void vc8x0_gpio_setup(struct vc8x0_dev *dev)
{
	switch (dev->board) {
	case VC8X0_BOARD_820E:
	case VC8X0_BOARD_820E_DEV:
	case VC8X0_BOARD_845E:
		break;
	}
}

void vc8x0_card_setup(struct vc8x0_dev *dev)
{
	switch (dev->board) {
	case VC8X0_BOARD_820E:
	case VC8X0_BOARD_820E_DEV:
	case VC8X0_BOARD_845E:
		break;
	}
}

