/*
 *  Driver for the ViewCast 845e PCIe Capture Card
 *
 *  Copyright 2011/2012, Kernel Labs Inc. www.kernellabs.com.
 *   - Steven Toth <stoth@kernellabs.com>
 *
 *  Project sponsored by ViewCast Texas, US.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* GN4124 registers BAR4 */
#define GN4124_PCIE_DCR		0x60

#define GN4124_PCI_SYS_CFG_SYSTEM 0x0800
#define GN4124_LB_CTL		0x0804
#define GN4124_CLK_CSR		0x0808
#define GN4124_INT_CTRL		0x0810
#define GN4124_INT_STAT		0x0814

/* GN4124_INT_STAT register bits */
#define INT_STAT_TWI	0x0001
#define INT_STAT_FCLI	0x0002
#define INT_STAT_SWI0	0x0004
#define INT_STAT_SWI1	0x0008
#define INT_STAT_INT0	0x0010
#define INT_STAT_INT1	0x0020
#define INT_STAT_INT2	0x0040
#define INT_STAT_INT3	0x0080
#define INT_STAT_ALI0	0x0100
#define INT_STAT_ALI1	0x0200
#define INT_STAT_ALI2	0x0400
#define INT_STAT_ALI3	0x0800
#define INT_STAT_ALI4	0x1000
#define INT_STAT_ALI5	0x2000
#define INT_STAT_ALI6	0x4000
#define INT_STAT_GPIO	0x8000
#define GN4124_INTERRPT_MASK 	(INT_STAT_GPIO | INT_STAT_ALI0 | INT_STAT_ALI1 | INT_STAT_ALI2 | INT_STAT_ALI3 | INT_STAT_ALI4 | INT_STAT_ALI5 | INT_STAT_ALI6)

#define GN4124_INT_CFG0		0x0820
#define GN4124_INT_CFG1		0x0824
#define GN4124_INT_CFG2		0x0828
#define GN4124_INT_CFG3		0x082C
#define GN4124_INT_CFG4		0x0830
#define GN4124_INT_CFG5		0x0834
#define GN4124_INT_CFG6		0x0838
#define GN4124_INT_CFG7		0x083C

#define GN4124_GPIO_BYPASS_MODE		0x0A00
#define GN4124_GPIO_DIRECTION_MODE	0x0A04
#define GN4124_GPIO_OUTPUT_ENABLE	0x0A08
#define GN4124_GPIO_OUTPUT_VALUE	0x0A0C
#define GN4124_GPIO_INPUT_VALUE		0x0A10
#define GN4124_GPIO_INT_MASK		0x0A14
#define GN4124_GPIO_INT_ENABLE		0x0A18
#define GN4124_GPIO_INT_DISABLE		0x0A1C
#define GN4124_GPIO_INT_STATUS		0x0A20
#define GN4124_GPIO_INT_TYPE		0x0A24
#define GN4124_GPIO_INT_VALUE		0x0A28
#define GN4124_GPIO_INT_ON_ANY		0x0A2C

#define GN4124_FCL_CTRL		0x0B00
#define GN4124_FCL_STATUS	0x0B04
#define GN4124_FCL_IODATA_IN	0x0B08
#define GN4124_FCL_IODATA_OUT	0x0B0C
#define GN4124_FCL_EN		0x0B10
#define GN4124_FCL_TIMER_0	0x0B14
#define GN4124_FCL_TIMER_1	0x0B18
#define GN4124_FCL_CLK_DIV	0x0B1C
#define GN4124_FCL_IRQ		0x0B20
#define GN4124_FCL_TIMER_CTRL	0x0B24
#define GN4124_FCL_IM		0x0B28
#define GN4124_FCL_TIMER2_0	0x0B2C
#define GN4124_FCL_TIMER2_1	0x0B30
#define GN4124_FCL_FIFO		0x0E00

#define GPIO_BIT_9		(0x0200)
#define GPIO_BIT_10		(0x0400)
#define GPIO_BIT_11		(0x0800)
#define GPIO_BIT_12		(0x1000)
#define GPIO_BIT_13		(0x2000)
#define GPIO_BIT_14		(0x4000)
#define GPIO_BIT_15		(0x8000)
#define DEBUG_LA_0              GPIO_BIT_9
#define DEBUG_LA_1              GPIO_BIT_10
#define DEBUG_LA_2              GPIO_BIT_11
#define DEBUG_LA_3              GPIO_BIT_12
#define DEBUG_LA_4              GPIO_BIT_13
#define DEBUG_LA_5              GPIO_BIT_14
#define DEBUG_LA_6              GPIO_BIT_15

/*
 * BAR0  Application Layer Registers
 * These are offsets based on 4 byte registers.
 */

#define DMA_STATUS_MASK     (0x0)
#define DMA_CFG             (0x4)
#define VDMA_EVENT_SET      (0x8)
#define VDMA_EVENT_CLR      (0xC)
#define VDMA_EVENT          (0x10)
#define VDMA_EVENT_EN       (0x14)
#define VDMA_SYS_ADDR_L     (0x18)
#define VDMA_SYS_ADDR_H     (0x1c)
#define VDMA_DPTR           (0x20)
#define VDMA_XFER_CTL       (0x24)
#define VDMA_REG_A          (0x28)
#define VDMA_REG_B          (0x2c)
#define VDMA_CSR            (0x30)
#define DMA_PAYLOAD_SIZE    (0x34)
#define DMA_STATUS          (0x38)
#define APP_STATUS_MASK     (0x50)
#define APP_CFG             (0x54)
#define GSPI_CTRL           (0x58)
#define GSPI_SDIN           (0x5C)
#define INIT_RD_WR_ADDR_DIFF (0x60)
#define APP_STATUS          (0x68)
#define APP_STATUS_RAW      (0x6C)

#define DPRAM               (0x4000)
#define DPRAM_INSTRUCTION   (DPRAM)

/* DMA_CFG register bits */
#define DMA_CFG_LDM_ENABLE	(0x0001)
#define DMA_CFG_PDM_ENABLE	(0x0002)
#define DMA_CFG_CSR_ENABLE	(0x0004)
#define DMA_CFG_LDM_FLUSH	(0x0008)
#define DMA_CFG_PDM_FLUSH	(0x0010)

#define DMA_CFG_LDM_WORD_BUFFER_EN (0x0800)

/* VDMA Event, enable, set, and clear register */
#define VDMA_EVENT_0        (0x0001)
#define VDMA_EVENT_1        (0x0002)
#define VDMA_EVENT_2        (0x0004)
#define VDMA_EVENT_3        (0x0008)
#define VDMA_EVENT_4        (0x0010)
#define VDMA_EVENT_5        (0x0020)
#define VDMA_EVENT_6        (0x0040)
#define VDMA_EVENT_7        (0x0080)
#define VDMA_EVENT_8        (0x0100)
#define VDMA_EVENT_9        (0x0200)
#define VDMA_EVENT_10       (0x0400)
#define VDMA_EVENT_11       (0x0800)
#define VDMA_EVENT_12       (0x1000)
#define VDMA_EVENT_13       (0x2000)
#define VDMA_EVENT_14       (0x4000)
#define VDMA_EVENT_15       (0x8000)
#define VDMA_EVENT_ALL      (0xFFFF)

/* VDMA Sequencer Register Bit Definitions */
#define VDMA_CSR_IPR        (0x0001)
#define VDMA_CSR_HALT       (0x0002)

/* DMA_STATUS_MASK register */
#define DMA_STATUS_MASK_PDM_IDLE (0x0001)
#define DMA_STATUS_MASK_LDM_IDLE (0x0002)
#define DMA_STATUS_MASK_PD_ERR   (0x0004)

/* APP_STATUS_MASK register */
#define APP_STATUS_MASK_FIFO_FULL   (0x0001)
#define APP_STATUS_MASK_FIFO_EMPTY  (0x0002) 
#define APP_STATUS_MASK_DVB         (0x0004)

/* APP_CFG register */
#define APP_CFG_CAPTURE         (0x0001)
#define APP_CFG_PLAYBACK        (0x0002)
#define APP_CFG_YUY2            (0x0000)
#define APP_CFG_UYVY            (0x0004)
#define APP_CFG_10BIT           (0x0008)
#define APP_CFG_DESERIALIER_CLK (0x0010)
#define APP_CFG_PROGRESSIVE     (0x0000)
#define APP_CFG_INTERLACE       (0x0020)

/* FPGA Registers BAR0 */
#define I2C_PRE_LO_REGISTER	(0x00)
#define I2C_PRE_HI_REGISTER	(0x08)

#define I2C_CONTROL_REGISTER	0x10
/* Control register and bit definitions */
#define I2C_BLOCK_ENABLE	0x80
#define I2C_INTR_ENABLE		0x40

#define I2C_TRANSMIT_REGISTER	0x18
#define I2C_RECEIVE_REGISTER	0x18

#define I2C_STATUS_REGISTER	0x20
/* Status register bits */
#define I2C_IN_PROGRESS		0x02
#define I2C_BUS_BUSY		0x40

#define I2C_COMMAND_REGISTER	0x20
/* Command register bits */
#define I2C_START		0x80
#define I2C_STOP		0x40
#define I2C_READ		0x20
#define I2C_WRITE		0x10
#define I2C_NACK		0x08
#define I2C_INTR_ACK		0x01

#define GSPI_WRITE_COMMAND       0x00000000
#define GSPI_READ_COMMAND        0x80000000
#define GSPI_READ_REGISTER       0x00
#define GSPI_WRITE_REGISTER      0x08
#define GSPI_STATUS_REGISTER     0x10
#define GSPI_CONTROL_REGISTER    0x18
#define GSPI_SLAVE_SEL_REGISTER  0x20
#define GSPI_DATA_READY_TO_READ  0xe0
#define GSPI_DATA_READY_TO_WRITE 0x60
#define GSPI_RETRY_COUNT         3

#define FPGA_SHA_EEPROM_DATA_REG_0      (0x40)
#define FPGA_SHA_EEPROM_DATA_REG_1      (0x44)
#define FPGA_SHA_EEPROM_DATA_REG_2      (0x48)
#define FPGA_SHA_EEPROM_DATA_REG_3      (0x4C)
#define FPGA_SHA_EEPROM_DATA_REG_4      (0x50)
#define FPGA_SHA_EEPROM_DATA_REG_5      (0x54)
#define FPGA_SHA_EEPROM_DATA_REG_6      (0x58)
#define FPGA_SHA_EEPROM_DATA_REG_7      (0x5C)
#define FPGA_SHA_SECURITY_UID_REG_1     (0x60)
#define FPGA_SHA_SECURITY_UID_REG_2     (0x64)
#define FPGA_SHA_HASH_REG_1             (0x68)
#define FPGA_SHA_HASH_REG_2             (0x6C)
#define FPGA_SHA_HASH_REG_3             (0x70)
#define FPGA_SHA_HASH_REG_4             (0x74)
#define FPGA_SHA_HASH_REG_5             (0x78)
#define FPGA_SHA_HASH_REG_6             (0x7C)
#define FPGA_SHA_CHALLENGE_REG_1        (0x80)
#define FPGA_SHA_CHALLENGE_REG_2        (0x84)

/* SHA Reg and Commands */
#define FPGA_SHA_COMMAND_REG            (0x88)
#define FPGA_SHA_RAND_CHLNG_CMD		0x01
#define FPGA_SHA_CLR_RAND_CHLNG_CMD     0x02
#define FPGA_SHA_HASH_COMPARE_CMD       0x04

/* SHA Status Reg and Status bits */
#define FPGA_SHA_STATUS_REG             (0x90)
#define FPGA_SHA_RAND_CHLNG_DONE        0x01
#define FPGA_SHA_HASH_COMPARE_DONE      0x07

